import static java.lang.Math.*;


public class vectorFunc{

    /*
    */
    
    public static vector add(vector a, vector b)
    {
        return new vector( a.x+b.x, a.y+b.y);
    }

    public static vector sub(vector a, vector b)
    {
        return new vector( a.x-b.x, a.y-b.y);
    }

    public static vector mul(vector a, vector b)
    {
        return new vector( a.x*b.x, a.y*b.y);
    }
    
    public static vector div(vector a, vector b)
    {
        return new vector( a.x/b.x, a.y/b.y);
    }
    
    public static vector neg(vector a)
    {
        return new vector( -a.x, -a.y);
    }
    
    /*
    */
    
    public static vector scale(vector a, float scalar)
    {
        return new vector( a.x*scalar, a.y*scalar);
    }
    
    public static vector invscale(vector a, float scalar) 
    {
        if (scalar != 0)
        {
            return new vector( a.x/scalar, a.y/scalar);
        }
        else
        {
            return new vector(a.x, a.y);
        }
    }
    
    public static vector invscale(vector a, double scalar) 
    {
        if (scalar != 0)
        {
            return new vector( a.x/scalar, a.y/scalar);
        }
        else
        {
            return new vector(a.x, a.y);
        }
    }
    
    public static vector scale(vector a, double scalar)
    {
        return new vector( a.x*scalar, a.y*scalar);
    }
    
    public static double dot(vector a, vector b)
    {
        return (a.x*b.x) + (a.y*b.y);
    }
    
    public static double cross(vector a, vector b)
    {
        return (a.x*b.y) - (a.y*b.x);
    }
    
    /*
    */
    
    public static double magSQ(vector a)
    {
        return (double)(pow(a.x,2.0) + pow(a.y,2.0));
    }
    
    public static double mag(vector a)
    {
        return (double)sqrt(magSQ(a));
    }
    
    public static vector normalize(vector a)
    {
        double scalar = mag(a);
        return new vector( a.x/scalar, a.y/scalar);
    }
    
    public static vector normalizeSQ(vector a)
    {
        double scalarSQ = magSQ(a);
        return new vector( (double)pow(a.x,2.0)/scalarSQ, (double)pow(a.y,2.0)/scalarSQ);
    }
    
    /*
    */
    
    public static vector rotate(vector a, vector r)
    {
        return new vector(a.x * r.y - a.y * r.x, a.x * r.x + a.y * r.y);
    }
    
    public static vector fromRadian(double radian)
    {
        return new vector( sin(radian), cos(radian));
    }
    
    public static double toRadian(vector a)
    {
        return (double)atan2(a.x,a.y);
    }
    
    /*
    */
    
    public static double closestU(vector a, vector p1, vector p2)
    {
        vector delta = sub(p2,p1);
        return dot(sub(a,p1),delta)/magSQ(delta);
    }
    
    public static vector closest(vector a, vector p1, vector p2, double U)
    {
        return add(p1, scale(sub(p2,p1),U));
    }
    
    public static vector closest(vector a, vector p1, vector p2)
    {
        vector delta = sub(p2,p1);
        double u = dot(sub(a,p1), delta)/ magSQ(delta);
        return add(p1, scale(delta,u));
    }
    
    /*
    */
    
    public static double parallelZero(vector p1, vector p2, vector p3, vector p4)
    {
        return cross(sub(p2,p1), sub(p4,p3));
    }
    
    public static double intersectUa(vector p1, vector p2, vector p3, vector p4, double pZero)
    {
        vector delta = sub(p4,p3);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static double intersectUb(vector p1, vector p2, vector p3, vector p4, double pZero)
    {
        vector delta = sub(p2,p1);
        vector deltaWTF = sub(p1,p3);
        return cross(delta,deltaWTF)/pZero;
    }
    
    public static vector intersect(vector p1, vector p2, vector p3, vector p4)
    {
        double pZero = parallelZero(p1,p2,p3,p4);
        double Ua = intersectUa(p1,p2,p3,p4,pZero);
        //double Ub = intersectUb(p1,p2,p3,p4,pZero);
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
    public static vector intersect(vector p1, vector p2, double Ua)
    {
        return add(p1,scale(sub(p2,p1),Ua));
    }
    
};
